package ut.maxorato.bsc.crawler;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import ut.maxorato.bsc.crawler.analysis.Analysis;
import ut.maxorato.bsc.crawler.split.LogSplitter;

import java.io.File;
import java.util.Scanner;

public class Main {
    private static final Logger log = LoggerFactory.getLogger(LogSplitter.class);

    public static void main(String[] args) {
        runCommand(args);
    }

    public static void runCommand(String[] args) {
        try {
            if (args.length == 0) {
                Scanner scanner = new Scanner(System.in);

                while (scanner.hasNextLine()) {
                    String[] inputArgs = scanner.nextLine().split(" ");

                    if (inputArgs[0].equals("quit")) {
                        break;
                    }

                    runCommand(inputArgs);
                }
            } else if (args[0].equals("help")) {
                System.out.println("Commands:");
                System.out.println("  analyze <file>");
                System.out.println("    Generates templates for the URLs listed in the file and generates new URLs based on those.");
                System.out.println("  analyze <directory>");
                System.out.println("    Does the above for all .txt files in the specified directory. Saves reports to results/<filename>.log");
                System.out.println("  split <file> <output>");
                System.out.println("    Reads URLs from a warc archive and writes the result in domain-named files in <output> directory.");
                System.out.println("  split <directory>");
                System.out.println("    Does the above for all .warc files in the specified directory.");
            } else if (args[0].equals("split")) {
                if (args.length < 3) {
                    log.error("Input file and output directory paths are missing.");
                    return;
                }

                LogSplitter logSplitter = new LogSplitter(new File(args[2]));
                File location = new File(args[1]);

                if (location.isFile()) {
                    logSplitter.splitLogFile(location);
                } else if (location.isDirectory()) {
                    for (File sub : location.listFiles()) {
                        if (sub.isFile() && sub.getName().endsWith(".warc")) {
                            logSplitter.splitLogFile(sub);
                        }
                    }
                }
            } else if (args[0].equals("analyze")) {
                if (args.length < 2) {
                    log.error("Must specify input file name.");
                    return;
                }

                File urlFile = new File(args[1]);
                if (urlFile.isFile()) {
                    File resultLocation = new File(urlFile.getParentFile(), "results/" + urlFile.getName() + ".log");
                    new Analysis(urlFile, resultLocation).run();
                } else if (urlFile.isDirectory()) {
                    for (File sub : urlFile.listFiles()) {

                        if (sub.isFile() && sub.getName().endsWith(".txt")) {
                            File resultLocation = new File(urlFile, "results/" + sub.getName() + ".log");
                            new Analysis(sub, resultLocation).run();
                        }
                    }
                }
            } else {
                System.err.println("Unknown command: " + args[0]);
            }
        } catch (Exception e) {
            System.err.println("Exception when running command.");
            e.printStackTrace();

            log.error("Exception when running command '{}'", args.length > 0 ? args[0] : "interactive");
        }
    }
}
