package ut.maxorato.bsc.crawler.analysis;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import ut.maxorato.bsc.crawler.model.AnalysisContext;
import ut.maxorato.bsc.crawler.model.UrlTemplate;
import ut.maxorato.bsc.crawler.model.UrlTemplateGroup;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public class GroupTemplateFilter implements Runnable {
    private static final Logger log = LoggerFactory.getLogger(GroupTemplateFilter.class);

    private final AnalysisContext context;
    private final TemplateFilter filter;

    public GroupTemplateFilter(AnalysisContext context, TemplateFilter filter) {
        this.context = context;
        this.filter = filter;
    }

    @Override
    public void run() {
        int initialTemplateCount = 0;
        int finalTemplateCount = 0;
        int initialGroupCount = context.templateGroups.size();

        for (Iterator<UrlTemplateGroup> iterator = context.templateGroups.values().iterator(); iterator.hasNext();) {
            UrlTemplateGroup group = iterator.next();
            List<UrlTemplate> accepted = new ArrayList<>();

            for (UrlTemplate template : group.templates.values()) {
                initialTemplateCount++;

                if (filter.accept(template)) {
                    accepted.add(template);
                    finalTemplateCount++;
                }
            }

            if (accepted.size() == 0) {
                iterator.remove();
            } else {
                group.templates.clear();

                for (UrlTemplate template : accepted) {
                    group.templates.put(template.key, template);
                }
            }
        }

        context.report.afterNumericFilterTemplateCount = finalTemplateCount;
        context.report.afterNumericFilterGroupCount = context.templateGroups.size();

        log.info("Changed from {} groups {} templates to {} groups {} templates with filter {}", initialGroupCount, initialTemplateCount, context.templateGroups.size(), finalTemplateCount, filter.getClass().getSimpleName());
    }
}
