package ut.maxorato.bsc.crawler.analysis;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import ut.maxorato.bsc.crawler.model.AnalysisContext;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;

public class UrlListReader implements Runnable {
    private static final Logger log = LoggerFactory.getLogger(UrlListReader.class);

    private final AnalysisContext context;
    private final File inputFile;

    public UrlListReader(AnalysisContext context, File inputFile) {
        this.context = context;
        this.inputFile = inputFile;
    }

    @Override
    public void run() {
        try (BufferedReader reader = new BufferedReader(new FileReader(inputFile))) {
            String line;

            while ((line = reader.readLine()) != null) {
                String trimmed = line.trim();
                if (trimmed.length() == 0) {
                    continue;
                }

                context.urls.add(trimmed);
            }

            reader.close();
        } catch (IOException e) {
            throw new RuntimeException(e);
        }

        context.report.urlCount = context.urls.size();

        log.info("Found {} URLs from file {}", context.urls.size(), inputFile.getAbsolutePath());
    }
}
