package ut.maxorato.bsc.crawler.diff;

import ut.maxorato.bsc.crawler.model.RequestContext;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class GeneratedUrlComparisonStrategy extends RemovedFixComparisonStrategy {
    private final Pattern redirectMatch;

    public GeneratedUrlComparisonStrategy(RequestContext requestContext, String originalUrl, String emptyPageUrl, Pattern redirectMatch) {
        super(requestContext, originalUrl, emptyPageUrl);
        this.redirectMatch = redirectMatch;
    }

    @Override
    protected boolean isValidRedirectUrl(String url) {
        if (super.isValidRedirectUrl(url)) {
            return true;
        }

        Matcher match = redirectMatch.matcher(url);
        return match.matches();
    }

    public boolean isValidOtherEntityUrl(String alternateUrl) {
        ComparisonContext context = new ComparisonContext(alternateUrl);

        Boolean initialResult = responseCodeBasedComparison(context);
        if (initialResult != null) {
            return initialResult;
        }

        loadEmptyResult();

        // If empty result means error code, then this is not empty result
        if (!emptyResult.isSuccess()) {
            return true;
        }

        loadEmptyPageDelta();
        initialiseContextResult(context);

        DomDelta emptyVsOther = domDeltaCalculator.calculate(emptyResult.text, context.alternateResult.text);
        emptyVsOther = filterEffectiveDelta(emptyVsOther);

        if (emptyVsOther.differences.size() == 0) {
            return false;
        }

        DomDelta originalVsOther = domDeltaCalculator.calculate(originalResult.text, context.alternateResult.text);
        originalVsOther = filterEffectiveDelta(originalVsOther);
        if (originalVsOther.differences.size() == 0) {
            return false;
        }

        originalVsOther = removePrefixes(originalVsOther, emptyVsOther);

        if (originalVsOther.differences.size() == 0) {
            return true;
        }

        return false;
    }
}
