package ut.maxorato.bsc.crawler.diff;

import org.apache.commons.io.IOUtils;
import org.apache.http.Header;
import org.apache.http.HttpResponse;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;

import java.io.IOException;

public class HttpQueryExecutor {
    private final CloseableHttpClient httpClient;
    private final RequestConfig requestConfig;

    public HttpQueryExecutor() {
        httpClient = HttpClientBuilder.create().disableRedirectHandling().build();
        requestConfig = RequestConfig.custom()
                .setConnectionRequestTimeout(1000)
                .setConnectTimeout(1000)
                .setSocketTimeout(1000)
                .build();
    }

    private String getResponseMimeType(HttpResponse response) {
        Header contentType = response.getEntity().getContentType();

        if(contentType == null) {
            return null;
        }
        else {
            return contentType.getValue().split(";")[0].trim();
        }
    }

    public String getResponseEncoding(HttpResponse response) {
        Header contentType = response.getEntity().getContentType();

        if(contentType == null) {
            return "UTF-8";
        }
        else {
            String[] parts = contentType.getValue().split(";");

            if (parts.length > 1) {
                return parts[1].split("=")[1].trim();
            }
            else {
                return "UTF-8";
            }
        }
    }

    public QueryResult execute(String url) {
        HttpGet request = new HttpGet(url);
        request.setConfig(requestConfig);

        try (CloseableHttpResponse response = httpClient.execute(request)) {
            int code = response.getStatusLine().getStatusCode();

            if (code == 301 || code == 302) {
                String redirectLocation = response.getFirstHeader("Location").getValue();
                return new QueryResult(code, null, redirectLocation, null);
            }

            String mimeType = getResponseMimeType(response);
            String encoding = getResponseEncoding(response);
            String responseText = IOUtils.toString(response.getEntity().getContent(), encoding);

            response.close();

            return new QueryResult(code, responseText, null, mimeType);
        } catch (IOException e) {
            return new QueryResult(0, null, null, null);
        }
    }
}
