package ut.maxorato.bsc.crawler.model;

import java.util.List;
import java.util.regex.Pattern;

public class UrlTemplate {
    public final String key;
    public final List<PathElement> elements;
    public int valueSetCount;
    public List<Integer> finalValues;
    public String urlWithPlaceholder;
    public int knownValue;

    public UrlTemplate(List<PathElement> initialPathElements) {
        this(initialPathElements, 1);
    }

    public UrlTemplate(List<PathElement> initialPathElements, int valueSetCount) {
        this.key = PathElement.buildKey(initialPathElements);
        this.elements = initialPathElements;
        this.valueSetCount = valueSetCount;
    }

    public void merge(UrlTemplate template) {
        merge(template.elements);

        valueSetCount += template.valueSetCount;
    }

    public void merge(List<PathElement> mergeElements) {
        for (int i = 0; i < elements.size(); i++) {
            PathElement element = elements.get(i);
            PathElement joinedElement = mergeElements.get(i);

            switch (element.type) {
                case NUM_VALUE:
                case TEXT_VALUE:
                    element.values.addAll(joinedElement.values);
                    break;
            }
        }
    }

    public String getFirstValue(int elementIndex) {
        return elements.get(elementIndex).values.get(0);
    }

    public String getGroupKey() {
        return PathElement.buildGroupKey(elements);
    }

    public String getKeyWithConstant(int index, String replace) {
        return PathElement.buildKeyWithConstant(elements, index, replace);
    }

    public String getKeyWithAllConstantExcept(int urlIndex, int elementIndex) {
        return PathElement.buildKeySingleVariable(elements, urlIndex, elementIndex, false);
    }

    public String nthUrlWithPlaceholder(int urlIndex, int elementIndex) {
        return PathElement.buildKeySingleVariable(elements, urlIndex, elementIndex, true);
    }

    public Pattern getRegexPattern() {
        StringBuilder regex = new StringBuilder("^");

        for (PathElement element : elements) {
            switch (element.type) {
                case DOMAIN:
                    if (element.values.get(0).startsWith("http://")) {
                        regex.append(Pattern.quote("http://"));
                    } else {
                        regex.append(Pattern.quote("https://"));
                    }
                    regex.append("[^?/]+");
                    break;
                case SEPARATOR:
                case KEY:
                case TEXT_STATIC:
                    regex.append(Pattern.quote(element.values.get(0)));
                    break;
                case NUM_VALUE:
                    regex.append("([0-9]+)");
                    break;
                case TEXT_VALUE:
                    regex.append("[^?/]+");
                    break;
            }
        }

        regex.append("$");
        return Pattern.compile(regex.toString());
    }
}
